package fr.free.supertos;

import java.util.Random;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.Window;
import android.view.WindowManager;


public class FloconsActivity extends Activity {
	
	private ProgressDialog progressDialog;
	Thread t;
	ThreadRenderView renderView;
	
	// Classe interne de gestion des messages en provenance des threads
    Handler handlerMsg= new Handler()
    {
		 
    	public void handleMessage(android.os.Message msg)
    	{
    		// Fin du thread de temporisation sur la dialogue
    		if(msg.what == 0) {
    			progressDialog.dismiss();
    		}
   		}
   	};

    @Override
    public void onCreate(Bundle savedInstanceState) {
    	
    	Log.i("flocons", "onCreate");
        super.onCreate(savedInstanceState);
                       
        // Desactiver la barre de titre de notre application
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        // Passer la fentre en full-creen == cacher la barre de notification
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
        WindowManager.LayoutParams.FLAG_FULLSCREEN);

        progressDialog = new ProgressDialog(this);
        
        progressDialog.setMessage("Flocons de neige !");
        // On affiche notre message
        progressDialog.show();
        // Lancement du thread de temporisation affichage dialogue
        t = new MonThread(handlerMsg);
        t.start();
        
    	renderView = new ThreadRenderView(this);
    	setContentView(renderView);
    }
    
    @Override
    public void onResume()
    {
    	Log.i("flocons", "onResume");
    	// Cration ou recrtion d'une instance pour le rendu 2D

    	super.onResume();
    	
    	renderView.onResume();
    }

    @Override
    public void onPause()
    {
    	Log.i("flocons", "onPause");
    	// Si le thread de tempo dialogue existe encore
    	if (t.isAlive())
    	{
    		// Le flinguer
    		t.interrupt();
    		// Et fermer ladialogue
    		progressDialog.dismiss();
    	}
    	
    	// Mme chose por le thread d'animation
    	renderView.onPause();	 	
	 	
    	super.onPause();
    }

    public class ThreadRenderView extends SurfaceView implements Runnable
    {
       	private final static int MAXFLOCON = 100;
    	Flocon [] flocon = new Flocon[MAXFLOCON];
    	Thread tAnim;
    	SurfaceHolder holder;
    	
    	private int nbFlocon=0;
		private boolean execute;
		private Thread renderThread;

		public ThreadRenderView(Context context) {
			super(context);
			
			holder = getHolder();
			// TODO Auto-generated constructor stub
		}

		@Override
        public void run() {
        	while (execute)
        	{
	        	if(holder.getSurface().isValid())
	        	{
	        		Canvas canvas = holder.lockCanvas();
		        	
		            Paint paint = new Paint();
		            
		            paint.setAntiAlias(true);
		            canvas.drawRGB(0, 0, 0);
		            
		            // Nous allons dessiner nos points par rapport  la rsolution de l'cran
		            int iWidth = canvas.getWidth(); // Largeur
		            int iHeight = canvas.getHeight(); // Hauteur
		            
		            // Un flocon est blanc
		            paint.setARGB(255, 255,255,255);
		            
		            Random rand = new Random();
		            
		            for (int i = 0; i< MAXFLOCON; i++)
		            {
		            	
		            	
		            	// Si pour cet lment du tableau, il n'y a pas d'instance de flocon
		            	if (flocon[i] == null)
			            {
		            		
		            		// Nous allons limiter le nb. de flocons  10 max. via nbFlocon
		            		// cela nous evitera de voir tomber tous les flocons d'un coup !
		            		// De plus, pour un flocon qui pourrait tomper, nous faisons encore fonctionner l'alatoire
		            		// pour voir s'il tombe effectivement !
		            		int dep = rand.nextInt(100);
		            		if (dep < 5 && nbFlocon < 10)
		            		{
		            			flocon[i] = new Flocon (iWidth, iHeight);
		            			nbFlocon++;
		            		}
			            }
		            	else
		            	{
		            		// Une instance existe, donc
		            		// Affichage du flocon de neige
			            	canvas.drawCircle(flocon[i].getX(), flocon[i].getY(), flocon[i].getTaille(), paint);
			            	flocon[i].tombeFlocon();// Faisons le tomber...
			            	if (flocon[i].enBas())
			            	{
			            		flocon[i] = null; // Si arriv en bas, fin de l'instance
			            	}
		            	}            	
		            }            
		            
		            // 10 Flocns max, compteur qui est lui mme soumis au hazard pour se dcrmenter !
		        	if (rand.nextInt(10) < 2)
		        		if (nbFlocon> 0)
		        			nbFlocon--;
		        	
		        	holder.unlockCanvasAndPost(canvas); // Ce qui provoquera l'affichage
		        	
		        	try {
						Thread.sleep(100);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						Thread.currentThread().interrupt(); // Marque le thread comme interrompu
						execute =false;
						return;
					}
	        	}
        	}
        }
        
        public void onPause()
        {
        	// Arrt du thread temporisation animation
        	execute = false;
        	renderThread.interrupt();
        }
        
        public void onResume()
        {
            // Lancement du Thread permettant de temporiser l'animation
        	execute = true; 
        	renderThread = new Thread(this);
            renderThread.start();
        }
    	
    }
    
    // Instance de thread permettent de temporiser l'affichage de notre dialogue  5 secondes
    public class MonThread extends Thread {
    	Handler handlerDial;
        public MonThread(Handler handlerDial) {
			// TODO Auto-generated constructor stub
        	this.handlerDial=handlerDial;
		}

		public void run() {
            // Traitement
            try {
            	Thread.sleep(5000);
            	handlerDial.sendEmptyMessage(0);
            } catch (InterruptedException ex) {
                Thread.currentThread().interrupt(); // Marque le thread comme interrompu
                return;
            }
        }
    }
	
}